@@ -1,4 +1,4 @@
-# alac 75 
+# alac 76
 # sam robson
 
 import glob
@@ -29,7 +29,7 @@ def write(conf, outputs):
     Writes outputs to path in conf
     """
     if not conf.DEBUG:
-        sys.tracebacklimit = 0
+        # sys.tracebacklimit = 0
         warnings.filterwarnings('ignore')
 
     if conf.OUTPUT_EXT == ".xls":
@@ -96,7 +96,6 @@ def archive(conf):
     start_time = time.time()
     from_archive = True if conf['IS_FULL_TEXT'] == True else False
     if not conf.DEBUG:
-        sys.tracebacklimit = 0
         warnings.filterwarnings('ignore')
 
     if conf.LOG or conf.DEBUG:
@@ -131,9 +130,9 @@ def archive(conf):
             outputs.to_pickle(conf.OUTPUT_PATH)
     if not conf.NO_WRITE and conf.OUTPUT_EXT == ".csv":
         if conf.COMPRESS:
-            outputs.to_csv(conf.OUTPUT_PATH + ".csv.zip", escapechar='\\',compression="zip")
+            outputs.to_csv(conf.OUTPUT_PATH + ".zip", escapechar='\\',compression="zip")
         else:
-            outputs.to_csv(conf.OUTPUT_PATH + ".csv", escapechar='\\')
+            outputs.to_csv(conf.OUTPUT_PATH, escapechar='\\')
     if not conf.NO_WRITE and conf.OUTPUT_EXT == ".parquet":
         if conf.COMPRESS:
             outputs.to_parquet(conf.OUTPUT_PATH + ".parquet", compression="brotli")
@@ -154,7 +153,6 @@ def init(conf):
     """
     a = []
     if not conf.DEBUG:
-        sys.tracebacklimit = 0
         warnings.filterwarnings('ignore')
     if conf.MAKE == "multiexport":
         a = cases(conf)
@@ -179,7 +177,6 @@ def table(conf):
     """
     a = []
     if not conf.DEBUG:
-        sys.tracebacklimit = 0
         warnings.filterwarnings('ignore')
     if conf.MAKE == "multiexport":
         a = cases(conf)
@@ -204,7 +201,6 @@ def fees(conf):
         'AmtPaid': '', 'Balance': '', 'AmtHold': ''})
     """
     if not conf.DEBUG:
-        sys.tracebacklimit = 0
         warnings.filterwarnings('ignore')
     queue = conf.QUEUE
     from_archive = True if conf.IS_FULL_TEXT else False
@@ -235,17 +231,21 @@ def fees(conf):
             b['CaseNumber'] = b['CaseInfoOutputs'].map(lambda x: x[0])
             b['FeeOutputs'] = b.index.map(lambda x: getFeeSheet(str(b.loc[x].AllPagesText)))
             feesheet = b['FeeOutputs'].map(lambda x: x[6])
-
+            try:
+                feesheet['AmtDue'] = feesheet['AmtDue'].map(lambda x: pd.to_numeric(x, 'coerce'))
+                feesheet['AmtPaid'] = feesheet['AmtPaid'].map(lambda x: pd.to_numeric(x, 'coerce'))
+                feesheet['Balance'] = feesheet['Balance'].map(lambda x: pd.to_numeric(x, 'coerce'))
+                feesheet['AmtHold'] = feesheet['AmtHold'].map(lambda x: pd.to_numeric(x, 'coerce'))
+            except:
+                pass
+            
             feesheet = feesheet.dropna()
             fees = fees.dropna()
             feesheet = feesheet.tolist()  # -> [df, df, df]
             feesheet = pd.concat(feesheet, axis=0, ignore_index=True)
             fees = fees.append(feesheet, ignore_index=True)
             fees.fillna('', inplace=True)
-            fees['AmtDue'] = fees['AmtDue'].map(lambda x: pd.to_numeric(x, 'coerce'))
-            fees['AmtPaid'] = fees['AmtPaid'].map(lambda x: pd.to_numeric(x, 'coerce'))
-            fees['Balance'] = fees['Balance'].map(lambda x: pd.to_numeric(x, 'coerce'))
-            fees['AmtHold'] = fees['AmtHold'].map(lambda x: pd.to_numeric(x, 'coerce'))
+
     if not conf.NO_WRITE:
         write(conf, fees)
     complete(conf)
@@ -258,7 +258,6 @@ def charges(conf):
     charges = pd.DataFrame({'CaseNumber': '', 'Num': '', 'Code': '', 'Felony': '', 'Conviction': '', 'CERV': '', 'Pardon': '', 'Permanent': '', 'Disposition': '', 'CourtActionDate': '', 'CourtAction': '', 'Cite': '', 'TypeDescription': '', 'Category': '', 'Description': ''})
     """
     if not conf.DEBUG:
-        sys.tracebacklimit = 0
         warnings.filterwarnings('ignore')
     queue = conf['QUEUE']
     conf.DEDUPE = conf['DEDUPE']
@@ -320,7 +319,6 @@ def cases(conf):
     See API docs for conf.TABLE specific outputs
     """
     if not conf.DEBUG:
-        sys.tracebacklimit = 0
         warnings.filterwarnings('ignore')
     queue = conf['QUEUE']
     arch = pd.DataFrame()
@@ -395,17 +393,21 @@ def cases(conf):
             chargetabs = pd.concat(chargetabs, axis=0, ignore_index=True)
 
             charges = charges.append(chargetabs, ignore_index=True)
-
-            feesheet['AmtDue'] = feesheet['AmtDue'].map(lambda x: pd.to_numeric(x, 'coerce'))
-            feesheet['AmtPaid'] = feesheet['AmtPaid'].map(lambda x: pd.to_numeric(x, 'coerce'))
-            feesheet['Balance'] = feesheet['Balance'].map(lambda x: pd.to_numeric(x, 'coerce'))
-            feesheet['AmtHold'] = feesheet['AmtHold'].map(lambda x: pd.to_numeric(x, 'coerce'))
-
-            b['ChargesTable'] = b['ChargesOutputs'].map(lambda x: x[-1])
-            b['Phone'] = b['Phone'].map(lambda x: pd.to_numeric(x, 'coerce'))
-            b['TotalAmtDue'] = b['TotalAmtDue'].map(lambda x: pd.to_numeric(x, 'coerce'))
-            b['TotalBalance'] = b['TotalBalance'].map(lambda x: pd.to_numeric(x, 'coerce'))
-            b['PaymentToRestore'] = b['TotalBalance'].map(lambda x: pd.to_numeric(x, 'coerce'))
+            try:
+                feesheet['AmtDue'] = feesheet['AmtDue'].map(lambda x: pd.to_numeric(x, 'coerce'))
+                feesheet['AmtPaid'] = feesheet['AmtPaid'].map(lambda x: pd.to_numeric(x, 'coerce'))
+                feesheet['Balance'] = feesheet['Balance'].map(lambda x: pd.to_numeric(x, 'coerce'))
+                feesheet['AmtHold'] = feesheet['AmtHold'].map(lambda x: pd.to_numeric(x, 'coerce'))
+            except:
+                pass
+            try:
+                b['ChargesTable'] = b['ChargesOutputs'].map(lambda x: x[-1])
+                b['Phone'] = b['Phone'].map(lambda x: pd.to_numeric(x, 'coerce'))
+                b['TotalAmtDue'] = b['TotalAmtDue'].map(lambda x: pd.to_numeric(x, 'coerce'))
+                b['TotalBalance'] = b['TotalBalance'].map(lambda x: pd.to_numeric(x, 'coerce'))
+                b['PaymentToRestore'] = b['TotalBalance'].map(lambda x: pd.to_numeric(x, 'coerce'))
+            except:
+                pass
 
             if bool(conf.OUTPUT_PATH) and len(conf.OUTPUT_EXT) > 2 and i > 0 and not conf.NO_WRITE:
                 if os.path.getsize(conf.OUTPUT_PATH) > 1000:
@@ -486,12 +488,12 @@ def cases(conf):
 
                 elif conf.OUTPUT_EXT == ".json":
                     if conf.COMPRESS:
-                        cases.to_json(conf.OUTPUT_PATH+".zip", orient='table', compression="zip")
+                        cases.to_json(conf.OUTPUT_PATH, orient='table', compression="zip")
                     else:
                         cases.to_json(conf.OUTPUT_PATH, orient='table')
                 elif conf.OUTPUT_EXT == ".csv":
                     if conf.COMPRESS:
-                        cases.to_csv(conf.OUTPUT_PATH+".zip", escapechar='\\', compression="zip")
+                        cases.to_csv(conf.OUTPUT_PATH, escapechar='\\', compression="zip")
                     else:
                         cases.to_csv(conf.OUTPUT_PATH, escapechar='\\')
                 elif conf.OUTPUT_EXT == ".md":
@@ -521,7 +523,6 @@ def caseinfo(conf):
     See API docs for table specific outputs
     """
     if not conf.DEBUG:
-        sys.tracebacklimit = 0
         warnings.filterwarnings('ignore')
     queue = conf['QUEUE']
     start_time = time.time()
@@ -580,11 +581,13 @@ def caseinfo(conf):
             b['FeeCodesOwed'] = b['FeeOutputs'].map(lambda x: x[3])
             b['FeeCodes'] = b['FeeOutputs'].map(lambda x: x[4])
             b['FeeSheet'] = b['FeeOutputs'].map(lambda x: x[5])
-
-            b['Phone'] = b['Phone'].map(lambda x: pd.to_numeric(x, 'coerce'))
-            b['TotalAmtDue'] = b['TotalAmtDue'].map(lambda x: pd.to_numeric(x, 'coerce'))
-            b['TotalBalance'] = b['TotalBalance'].map(lambda x: pd.to_numeric(x, 'coerce'))
-            b['PaymentToRestore'] = b['TotalBalance'].map(lambda x: pd.to_numeric(x, 'coerce'))
+            try:
+                b['Phone'] = b['Phone'].map(lambda x: pd.to_numeric(x, 'coerce'))
+                b['TotalAmtDue'] = b['TotalAmtDue'].map(lambda x: pd.to_numeric(x, 'coerce'))
+                b['TotalBalance'] = b['TotalBalance'].map(lambda x: pd.to_numeric(x, 'coerce'))
+                b['PaymentToRestore'] = b['TotalBalance'].map(lambda x: pd.to_numeric(x, 'coerce'))
+            except:
+                pass
 
             if bool(conf.OUTPUT_PATH) and len(conf.OUTPUT_EXT) > 2 and i > 0 and not conf.NO_WRITE:
                 if os.path.getsize(conf.OUTPUT_PATH) > 1000:
@@ -617,7 +620,11 @@ def caseinfo(conf):
                    inplace=True)
 
             if conf.DEDUPE:
+                oldlen = cases.shape[0]
                 cases = cases.drop_duplicates()
+                newlen = cases.shape[0]
+                if newlen < oldlen:
+                    click.echo_yellow(f"Removed {oldlen-newlen} duplicate cases from write queue.")
 
             b.fillna('', inplace=True)
             cases = cases.append(b, ignore_index=True)
@@ -682,17 +689,16 @@ def map(conf, *args):
     Custom Parsing
     From config object and custom getter functions defined like below:
 
-    def getter(text: str):
+    def getter(full_case_text: str):
         out = re.search(...)
         ...
-        return str(out)
+        return out
 
-    Creates DataFrame with column for each getter column output and row for each case in queue
+    Creates DataFrame with column applying each getter function to every case in queue
 
     """
 
     if not conf.DEBUG:
-        sys.tracebacklimit = 0
         warnings.filterwarnings('ignore')
 
     if conf.DEDUPE:
@@ -702,7 +708,8 @@ def map(conf, *args):
         if dif > 0 and conf.LOG:
             click.secho(f"Removed {dif} duplicate cases from queue.", fg='bright_yellow', bold=True)
 
-    batches = batcher(conf)
+    if not conf.NO_BATCH:
+        batches = batcher(conf)
 
     start_time = time.time()
     func = pd.Series(args).map(lambda x: 1 if inspect.isfunction(x) else 0)
@@ -782,7 +789,6 @@ def setinputs(path, debug=False):
     pickle = None
     if not debug:
         warnings.filterwarnings('ignore')
-        sys.tracebacklimit = 0
 
     if isinstance(path, pd.core.frame.DataFrame) or isinstance(path, pd.core.series.Series):
         if "AllPagesText" in path.columns and path.shape[0] > 0:
@@ -800,7 +806,7 @@ def setinputs(path, debug=False):
             if queue.shape[0] > 0:
                 found = len(queue)
                 good = True
-        elif os.path.isfile(path) and os.path.splitext(path)[1] == ".xz":  # if archive -> good
+        elif os.path.isfile(path) and os.path.splitext(path)[1] == ".xz": 
             good = True
             pickle = pd.read_pickle(path, compression="xz")
             queue = pickle['AllPagesText']
@@ -809,7 +815,8 @@ def setinputs(path, debug=False):
         elif os.path.isfile(path) and (os.path.splitext(path)[1] == ".zip"):
             nzpath = path.replace(".zip","")
             nozipext = os.path.splitext(nzpath)[1]
-            logdebug(nozipext)
+            if debug:
+                click.echo(f"NZPATH: {nozipext}, NOZIPEXT: {nozipext}, PATH: {path}")
             if nozipext == ".json":
                 pickle = pd.read_json(path, orient='table',compression="zip")
                 queue = pickle['AllPagesText']
@@ -884,6 +891,11 @@ def setinputs(path, debug=False):
     })
     return out
 
+# alias for scrape - requires google chrome! must run attended! 
+def scrape(listpath, path, cID, uID, pwd, archive_path, qmax, qskip, speed, no_log):
+    from alacorder import scrape
+    scrape.go(listpath=listpath, path=path, cID=cID, uID=uID, pwd=pwd, archive_path=archive_path, qmax=qmax, qskip=qskip, speed=speed, no_log=no_log)
+
 
 def setoutputs(path="", debug=False, archive=False,table=""):
     good = False
@@ -894,7 +906,7 @@ def setoutputs(path="", debug=False, archive=False,table=""):
     ext = ""
     if not debug:
         warnings.filterwarnings('ignore')
-        sys.tracebacklimit = 0
+        # sys.tracebacklimit = 0
 
     if ".zip" in path or ".xz" in path:
         compress=True
@@ -950,6 +962,8 @@ def setoutputs(path="", debug=False, archive=False,table=""):
                 f"Alacorder failed to configure output! Try again with a valid path to a file with a supported extension, or run 'python -m alacorder --help' in command line for help.",
                 fg='red', bold=True)
 
+
+
     out = pd.Series({
         'OUTPUT_PATH': nzpath,
         'ZIP_OUTPUT_PATH': path,
@@ -963,7 +977,7 @@ def setoutputs(path="", debug=False, archive=False,table=""):
     return out
 
 
-def set(inputs, outputs=None, count=0, table='', overwrite=False, log=True, dedupe=False, no_write=False, no_prompt=False, skip_echo=False, debug=False, no_batch=False, compress=False):
+def set(inputs, outputs=None, count=0, table='', overwrite=False, log=True, dedupe=False, no_write=False, no_prompt=False, debug=False, no_batch=False, compress=False):
 
     status_code = []
     echo = ""
@@ -971,7 +985,7 @@ def set(inputs, outputs=None, count=0, table='', overwrite=False, log=True, dedu
     good = True
 
     if not debug:
-        sys.tracebacklimit = 0
+        # sys.tracebacklimit = 0
         warnings.filterwarnings('ignore')
 
     ## DEDUPE
@@ -979,9 +993,8 @@ def set(inputs, outputs=None, count=0, table='', overwrite=False, log=True, dedu
     if dedupe:
         queue = inputs.QUEUE.drop_duplicates()
         dif = content_len - queue.shape[0]
-        if debug:
+        if (log or debug) and dif > 0:
             click.secho(f"Removed {dif} duplicate cases from queue.", fg='bright_yellow', bold=True)
-            click.secho(queue)
     else:
         queue = inputs.QUEUE
 
@@ -992,6 +1005,8 @@ def set(inputs, outputs=None, count=0, table='', overwrite=False, log=True, dedu
         queue = inputs.QUEUE[0:ind]
     elif count > content_len and content_len > 0:
         count = inputs.QUEUE.shape[0]
+    elif count < content_len and count == 0:
+        count = content_len
     else:
         queue = inputs.QUEUE
 
@@ -1043,7 +1058,7 @@ def batcher(conf):
             batchsize = 2500
         else:
             batchsize = 1000
-        batches = np.array_split(q, math.floor(conf.FOUND / batchsize))
+        batches = np.array_split(q, 3)
     else:
         batches = np.array_split(q, 1)
     return batches
@@ -1051,9 +1066,9 @@ def batcher(conf):
 
 # same as calling set(setinputs(path), setoutputs(path), **kwargs)
 def setpaths(input_path, output_path=None, count=0, table='', overwrite=False, log=True, dedupe=False,
-             no_write=False, no_prompt=False, skip_echo=False, debug=False, no_batch=False, compress=False):
+             no_write=False, no_prompt=False, debug=False, no_batch=False, compress=False):
     if not debug:
-        sys.tracebacklimit = 0
+        # sys.tracebacklimit = 0
         warnings.filterwarnings('ignore')
     a = setinputs(input_path)
     if log:
@@ -1069,10 +1084,25 @@ def setpaths(input_path, output_path=None, count=0, table='', overwrite=False, l
         click.secho(c.ECHO)
     return c
 
-def setinit(input_path, output_path=None, archive=False,count=0, table='', overwrite=False, log=True, dedupe=False, no_write=False, no_prompt=False, debug=False, no_batch=False, compress=False):
-    a = setpaths(input_path=input_path, output_path=output_path, archive=archive, count=count, table=table, overwrite=overwrite, log=log, dedupe=dedupe, no_write=no_write, no_prompt=no_prompt,debug=debug, no_batch=no_batch, compress=compress)
-    b = init(a)
-    return b
+def setinit(input_path, output_path=None, archive=False,count=0, table='', overwrite=False, log=True, dedupe=False, no_write=False, no_prompt=False, debug=False, no_batch=False, compress=False, scrape=False, scrape_cID="",scrape_uID="", scrape_pwd="", scrape_qmax=0, scrape_qskip=0, scrape_speed=1):
+
+    if scrape:
+        scrape_no_log = not log
+        scrape.go(input_path, output_path, scrape_cID, scrape_uID, scrape_pwd, scrape_qmax, scrape_qskip, scrape_speed, scrape_no_log)
+    else:
+        if not isinstance(input_path, pd.core.series.Series) and input_path != None:
+            input_path = setinputs(input_path)
+
+        if not isinstance(output_path, pd.core.series.Series) and output_path != None:
+            output_path = setoutputs(output_path)
+
+        a = set(input_path, output_path, count=count, table=table, overwrite=overwrite, log=log, dedupe=dedupe, no_write=no_write, no_prompt=no_prompt,debug=debug, no_batch=no_batch, compress=compress)
+        
+        if archive == True:
+            a.MAKE = "archive"
+        
+        b = init(a)
+        return b
 
 ## GETTERS
 
@@ -1383,10 +1413,12 @@ def getTotals(text: str):
         tdue = totalrow.split("$")[1].strip().replace("$", "").replace(",", "").replace(" ", "")
         tpaid = totalrow.split("$")[2].strip().replace("$", "").replace(",", "").replace(" ", "")
         thold = totalrow.split("$")[4].strip().replace("$", "").replace(",", "").replace(" ", "")
-        tdue = pd.to_numeric(tdue, 'coerce')
-        tpaid = pd.to_numeric(tpaid, 'coerce')
-        thold = pd.to_numeric(thold, 'coerce')
-
+        try:
+            tdue = pd.to_numeric(tdue, 'coerce')
+            tpaid = pd.to_numeric(tpaid, 'coerce')
+            thold = pd.to_numeric(thold, 'coerce')
+        except:
+            pass
     except IndexError:
         totalrow = 0
         tdue = 0
@@ -1848,20 +1880,37 @@ def echo_conf(input_path, make, output_path, overwrite, no_write, dedupe, no_pro
     return d + e + f
 
 upick_table = ('''
-Select preferred table output below.
-    A:  Case Details
-    B:  Fee Sheets
-    C:  Charges (all)
-    D:  Charges (disposition only)
-    E:  Charges (filing only)
 
-Enter A, B, C, D, or E to continue.
+For compressed archive, enter:
+    [A] Full text archive
+
+To export a data table, enter:
+    [B]  Case Details
+    [C]  Fee Sheets
+    [D]  Charges (all)
+    [E]  Charges (disposition only)
+    [F]  Charges (filing only)
+
+Enter selection to continue. [A-F]
 ''')
 
+upick_table_only = ('''
+
+To export a data table, enter:
+    [B]  Case Details
+    [C]  Fee Sheets
+    [D]  Charges (all)
+    [E]  Charges (disposition only)
+    [F]  Charges (filing only)
+
+Enter selection to continue. [B-F]
+''')
 
 def pick_table():
     return click.style(upick_table)
 
+def pick_table_only():
+    return click.style(upick_table_only)
 
 ujust_table = ('''
 
@@ -1877,11 +1926,11 @@ def just_table():
 
 uboth = ('''
 
-EXPORT FULL TEXT ARCHIVE: To process case inputs into a full text archive (recommended), enter archive path below with file extension .pkl.xz.
+EXPORT FULL TEXT ARCHIVE: To process case inputs into a full text archive (recommended), enter archive path below with file extension .pkl.xz or .json.zip.
 
 EXPORT DATA TABLE: To export data table from case inputs, enter full output path. Use .xls or .xlsx to export all tables, or, if using another format (.csv, .json, .dta), select a table after entering output file path.
 
-Enter path.
+Enter output path.
 ''')
 
 
@@ -1889,14 +1938,13 @@ def both():
     return click.style(uboth)
 
 
-utitle = click.style("\nALACORDER beta 75",bold=True,italic=True) + """
+utitle = click.style("\nALACORDER beta 76",bold=True,italic=True) + """
 
 Alacorder processes case detail PDFs into data tables suitable for research purposes. Alacorder also generates compressed text archives from the source PDFs to speed future data collection from the same set of cases.
 
     ACCEPTED      /pdfs/path/   PDF directory           
     INPUTS:       .pkl.xz       Compressed pickle archive
-                  .json(.zip)   JSON (-zip flag to compress)
-
+                  .json.zip     Compressed JSON archive
 
 Enter input path.
 """
@@ -1905,6 +1953,14 @@ Enter input path.
 def title():
     return utitle
 
+usmalltitle = click.style("\nALACORDER beta 76",bold=True,italic=True) + """
+
+Alacorder processes case detail PDFs into data tables suitable for research purposes. Alacorder also generates compressed text archives from the source PDFs to speed future data collection from the same set of cases.
+
+"""
+
+def smalltitle():
+    return title
 
 utext_p = ('''
 
@@ -1926,7 +1982,7 @@ def complete(conf, *outputs):
     elapsed = math.floor(time.time() - conf.TIME)
 
     if not conf.DEBUG:
-        sys.tracebacklimit = 0
+        # sys.tracebacklimit = 0
         warnings.filterwarnings('ignore')
     if conf.LOG and len(outputs) > 0:
         click.secho(outputs)
@@ -1935,7 +1991,7 @@ def complete(conf, *outputs):
 
 
 def logdebug(conf, *msg):
-    if conf['DEBUG']:
+    if conf.DEBUG:
         click.secho(msg)
 
 
